#include "stdafx.h"
#include <stdio.h>
#include <windows.h>
#include "MyRegistre.h"
#include "MyException.h"

// *************************************************************************************

void CMyRegistre::SetAutoCreate(BOOL autoCreate)
{
	m_isAutoCreate = autoCreate;
}

// *************************************************************************************

//ex.:"HKEY_CURRENT_USER\Software\PlayListManager"
char * NextSubKey(CMyString &key,CMyString *subKey)
{
	for (int i=0;i<key.size;i++)
	{
		if (key[i]=='\\')
		{
			key[i]='\0';
			*subKey = key;
			key[i]='\\';
			return key.value+i+1;
		}
	}
	return NULL;
}

void HKeyToString (HKEY hKey,CMyString &result)
{
	switch ((long)hKey)
	{
		case HKEY_CLASSES_ROOT: result="HKEY_CLASSES_ROOT"; break;
		case HKEY_CURRENT_CONFIG: result="HKEY_CURRENT_CONFIG";break;
		case HKEY_CURRENT_USER: result="HKEY_CURRENT_USER"; break;
		case HKEY_LOCAL_MACHINE: result="HKEY_LOCAL_MACHINE"; break;
		case HKEY_USERS: result="HKEY_USERS";break;
		case HKEY_PERFORMANCE_DATA: result="HKEY_PERFORMANCE_DATA";break;
		case HKEY_DYN_DATA: result="HKEY_DYN_DATA"; break;
		default: result="";
	}
}

// *************************************************************************************

CMyRegistre::CMyRegistre(BOOL autoCreate)
{
	m_isAutoCreate = autoCreate;
	m_isValideKey = FALSE;
}

// *************************************************************************************

BOOL CMyRegistre::operator= (char *nameKey)				
{
	return Read(nameKey);
}

// *************************************************************************************

BOOL CMyRegistre::PrefixKey (char *nameKey)
{
	CMyString fullNameKey = nameKey;
	CMyString subKey;
	char *nextKey = NextSubKey(fullNameKey,&subKey);
	if (nextKey)
	{
		if (subKey=="HKEY_CLASSES_ROOT" || subKey=="HKCR")			
			m_hKey=HKEY_CLASSES_ROOT;
		else if (subKey=="HKEY_CURRENT_CONFIG" || subKey=="HKLM")	
			m_hKey=HKEY_CURRENT_CONFIG;
		else if (subKey=="HKEY_CURRENT_USER" || subKey=="HKCU")		
			m_hKey=HKEY_CURRENT_USER;
		else if (subKey=="HKEY_LOCAL_MACHINE" || subKey=="HKLM")	
			m_hKey=HKEY_LOCAL_MACHINE;
		else if (subKey=="HKEY_USERS" || subKey=="HKU")				
			m_hKey=HKEY_USERS;
		else if (subKey=="HKEY_PERFORMANCE_DATA" || subKey=="HKPD")	
			m_hKey=HKEY_PERFORMANCE_DATA;
		else if (subKey=="HKEY_DYN_DATA" || subKey=="HKDD")			
			m_hKey=HKEY_DYN_DATA;
		else 
		{
			m_hKey = HKEY_CURRENT_USER;
			return TRUE;
		}
		m_nameKey = nextKey;
		return TRUE;
	}
	m_hKey = NULL;
	return FALSE;
}

// *****************************************************************************************

BOOL CMyRegistre::Read(char *nameKey)
{
	if (!PrefixKey(nameKey)) return FALSE;
	HKEY hKey;
	if (RegOpenKeyExA( m_hKey, m_nameKey , 0 , KEY_READ, &hKey)!=ERROR_SUCCESS) 
	{
		if (m_isAutoCreate) 
			return RecursiveWrite(nameKey);
		return FALSE;
	}
	RegCloseKey (hKey) ;
	m_isValideKey = TRUE;
	return TRUE;
}

// *****************************************************************************************

BOOL CMyRegistre::RecursiveWrite(char *nameKey)
{
	if (!PrefixKey(nameKey)) return FALSE;
	char *next;
	CMyString fullKey;
	CMyString keyCur;
	CMyString key;
	CMyString subKey;
	HKeyToString(m_hKey,fullKey);
	key = m_nameKey;
	BOOL lastWrite;
	while (next = NextSubKey(key,&subKey))
	{
		keyCur = next;
		key = keyCur;

		CMyRegistre reg;
		reg.Write(fullKey,subKey);
		fullKey<<"\\"<<subKey;
	}
	CMyRegistre reg;
	lastWrite = reg.Write(fullKey,key);

	m_isValideKey = lastWrite;
	return lastWrite;
}

// *****************************************************************************************

BOOL CMyRegistre::Write(char *nameKey,char *subKey)
{
	m_isValideKey = FALSE;
	if (!PrefixKey(nameKey)) return FALSE;
	HKEY hKey,hKeySub; 
	if (RegOpenKeyExA( m_hKey, m_nameKey , 0 , KEY_WRITE, &hKey)!=ERROR_SUCCESS) return FALSE;
	if (RegCreateKeyA(hKey,subKey,&hKeySub) != ERROR_SUCCESS) 
	{
		RegCloseKey (hKey);
		return FALSE;
	}
	RegCloseKey (hKeySub);
	m_isValideKey = TRUE;
	return TRUE;
}

// *************************************************************************************

BOOL CMyRegistre::GetValue(char *param , char *result)
{
	if (!m_isValideKey) return FALSE;
	HKEY hKey ; 
	DWORD dwSize=MAX_REGISTRY_SIZE;
	ULONG uType = REG_SZ ;

	if (RegOpenKeyExA(m_hKey,m_nameKey,0,KEY_QUERY_VALUE,&hKey) != ERROR_SUCCESS) return FALSE;
	if (RegQueryValueExA (hKey,param,NULL,&uType,(unsigned char *)result,&dwSize) != ERROR_SUCCESS) return FALSE;
	RegCloseKey (hKey) ;
	return TRUE;
}

/* *****************************************************************************************
   Place la valeur value dans le parametre param de la base de registre selectionner par nameKey
***************************************************************************************** */

BOOL CMyRegistre::SetValue(char *param,char *value)
{
	if (!m_isValideKey) return FALSE;
	HKEY hKey ; 
	if (RegOpenKeyExA  (m_hKey,m_nameKey,0,KEY_SET_VALUE,&hKey) != ERROR_SUCCESS) return FALSE;
    if (RegSetValueExA (hKey,param,NULL,REG_SZ,(unsigned char *)value,strlen(value)) != ERROR_SUCCESS) return FALSE;
	RegCloseKey (hKey) ;
	return TRUE;
}

// *************************************************************************************

int CMyRegistre::GetValue(char *param, CMyString &result)
{
	if (!m_isValideKey) return FALSE;
	result.Resize(4096);
	int ret = GetValue(param,result.value);
	result.Resize();
	return ret;
}

// *************************************************************************************

int CMyRegistre::GetValue(char *param,int *result)
{
	return GetValue(param,(long *)result);
}

int CMyRegistre::GetValue(char *param,long *result)
{
	if (!m_isValideKey) return FALSE;
	HKEY hKey ; 
	DWORD dwSize=sizeof(REG_DWORD);
	ULONG uType = REG_DWORD;;

	if (RegOpenKeyExA(m_hKey,m_nameKey,0,KEY_QUERY_VALUE ,&hKey) != ERROR_SUCCESS) return FALSE;
	if (RegQueryValueExA (hKey,param,NULL,&uType, (unsigned char *)result, &dwSize) != ERROR_SUCCESS) return FALSE;
	
	RegCloseKey (hKey) ;
	return TRUE;
}

// *************************************************************************************

int CMyRegistre::SetValue(char *param,int value)
{
	return SetValue(param,(long)value);
}

// *************************************************************************************

int CMyRegistre::SetValue(char *param,long value)
{
	if (!m_isValideKey) return FALSE;
	HKEY hKey ; 
	
	if (RegOpenKeyExA(m_hKey,m_nameKey,0,KEY_SET_VALUE,&hKey) != ERROR_SUCCESS) return FALSE;
    if (RegSetValueExA (hKey, param,NULL, REG_DWORD , (unsigned char *)&value, sizeof(REG_DWORD)) != ERROR_SUCCESS) return FALSE;
	RegCloseKey (hKey) ;
	return TRUE;
}

// *************************************************************************************

void CMyRegistre::BeginEnumKey ()
{
	m_numEnumKey = 0;
}

// *************************************************************************************

BOOL CMyRegistre::NextEnumKey(char *result)
{
	if (!m_isValideKey) return FALSE;
	HKEY hKey ; 	
	if (RegOpenKeyExA(m_hKey,m_nameKey,0,KEY_READ,&hKey) != ERROR_SUCCESS) return FALSE;
	if (RegEnumKeyA(hKey,m_numEnumKey++,result,MAX_REGISTRY_SIZE) != ERROR_SUCCESS) 
	{
		strcpy(result,"");
		return FALSE;
	}
	return TRUE;
}

// *************************************************************************************

void CMyRegistre::BeginEnumValue ()
{
	m_numEnumValue = 0;
}

// *************************************************************************************

BOOL CMyRegistre::NextEnumValue (char *result)
{
	if (!m_isValideKey) return FALSE;
	HKEY hKey ; 
	DWORD size = MAX_REGISTRY_SIZE;
	DWORD type;
	
	if (RegOpenKeyExA(m_hKey,m_nameKey,0,KEY_READ,&hKey) != ERROR_SUCCESS) return FALSE;
	if (RegEnumValueA(hKey,m_numEnumValue++,result,&size,NULL,&type,NULL,NULL) != ERROR_SUCCESS) 
	{
		strcpy(result,"");		
		return FALSE;
	}
	return TRUE;
}

/* *************************************************************************************
	Ex.:
	CMyRegitre reg("mySoftware");
	CMyString string = reg("myParam1","getstring");
	reg("myParam2","setint")=12;
************************************************************************************* */
/*
CMyString & CMyRegistre::operator() (char *param,char *typeAction);
{
	CMyString temp = typeAction;
	int interger;

	if (temp=="rint") 
	{ 
		GetValue(param,&integer); 
		m_result = interger; 
		return m_result; 
	}
	else if (temp=="rstring") 
	{ 
		GetValue(param,m_result); 
		return m_result; 
	}

	else if (temp=="wint") 
	{ 
		SetValue(param,integer); 
	}
}
*/